@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 
+import six
 import sys
 sys.path[0:0] = [""]
 
@@ -19,17 +20,12 @@ import uuid
 # websocket-client
 import websocket as ws
 
-# There are three tests to test the library's behaviour when the received
-# message is fragmented (please see [1] for details). They fail currently as
-# the library doesn't yet support fragmentation. Therefore the tests are
-# skipped unless the flag below is set.
-#
-# [1]: https://tools.ietf.org/html/rfc6455#section-5.4
-#      "RFC6455: 5.4. Fragmentation"
-#
-TEST_FRAGMENTATION = True
+# Skip test to access the internet.
 TEST_WITH_INTERNET = False
 
+# Skip Secure WebSocket test.
+TEST_SECURE_WS = False
+
 TRACABLE = False
 
 
@@ -64,8 +60,8 @@ class HeaderSockMock(SockMock):
     def __init__(self, fname):
         SockMock.__init__(self)
         path = os.path.join(os.path.dirname(__file__), fname)
-        self.add_packet(open(path).read())
-
+        with open(path, "rb") as f:
+            self.add_packet(f.read())
 
 class WebSocketTest(unittest.TestCase):
     def setUp(self):
@@ -75,96 +71,96 @@ class WebSocketTest(unittest.TestCase):
         pass
 
     def testDefaultTimeout(self):
-        self.assertEquals(ws.getdefaulttimeout(), None)
+        self.assertEqual(ws.getdefaulttimeout(), None)
         ws.setdefaulttimeout(10)
-        self.assertEquals(ws.getdefaulttimeout(), 10)
+        self.assertEqual(ws.getdefaulttimeout(), 10)
         ws.setdefaulttimeout(None)
 
     def testParseUrl(self):
         p = ws._parse_url("ws://www.example.com/r")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 80)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 80)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com/r/")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 80)
-        self.assertEquals(p[2], "/r/")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 80)
+        self.assertEqual(p[2], "/r/")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com/")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 80)
-        self.assertEquals(p[2], "/")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 80)
+        self.assertEqual(p[2], "/")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 80)
-        self.assertEquals(p[2], "/")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 80)
+        self.assertEqual(p[2], "/")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com:8080/r")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com:8080/")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://www.example.com:8080")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("wss://www.example.com:8080/r")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], True)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], True)
 
         p = ws._parse_url("wss://www.example.com:8080/r?key=value")
-        self.assertEquals(p[0], "www.example.com")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/r?key=value")
-        self.assertEquals(p[3], True)
+        self.assertEqual(p[0], "www.example.com")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/r?key=value")
+        self.assertEqual(p[3], True)
 
         self.assertRaises(ValueError, ws._parse_url, "http://www.example.com/r")
 
         p = ws._parse_url("ws://[2a03:4000:123:83::3]/r")
-        self.assertEquals(p[0], "2a03:4000:123:83::3")
-        self.assertEquals(p[1], 80)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "2a03:4000:123:83::3")
+        self.assertEqual(p[1], 80)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("ws://[2a03:4000:123:83::3]:8080/r")
-        self.assertEquals(p[0], "2a03:4000:123:83::3")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], False)
+        self.assertEqual(p[0], "2a03:4000:123:83::3")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], False)
 
         p = ws._parse_url("wss://[2a03:4000:123:83::3]/r")
-        self.assertEquals(p[0], "2a03:4000:123:83::3")
-        self.assertEquals(p[1], 443)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], True)
+        self.assertEqual(p[0], "2a03:4000:123:83::3")
+        self.assertEqual(p[1], 443)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], True)
 
         p = ws._parse_url("wss://[2a03:4000:123:83::3]:8080/r")
-        self.assertEquals(p[0], "2a03:4000:123:83::3")
-        self.assertEquals(p[1], 8080)
-        self.assertEquals(p[2], "/r")
-        self.assertEquals(p[3], True)
+        self.assertEqual(p[0], "2a03:4000:123:83::3")
+        self.assertEqual(p[1], 8080)
+        self.assertEqual(p[2], "/r")
+        self.assertEqual(p[3], True)
 
     def testWSKey(self):
         key = ws._create_sec_websocket_key()
-        self.assert_(key != 24)
-        self.assert_("¥n" not in key)
+        self.assertTrue(key != 24)
+        self.assertTrue(six.u("¥n") not in key)
 
     def testWsUtils(self):
         sock = ws.WebSocket()
@@ -175,32 +171,32 @@ class WebSocketTest(unittest.TestCase):
             "connection": "upgrade",
             "sec-websocket-accept": "Kxep+hNu9n51529fGidYu7a3wO0=",
             }
-        self.assertEquals(sock._validate_header(required_header, key), True)
+        self.assertEqual(sock._validate_header(required_header, key), True)
 
         header = required_header.copy()
         header["upgrade"] = "http"
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
         del header["upgrade"]
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
 
         header = required_header.copy()
         header["connection"] = "something"
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
         del header["connection"]
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
 
         header = required_header.copy()
         header["sec-websocket-accept"] = "something"
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
         del header["sec-websocket-accept"]
-        self.assertEquals(sock._validate_header(header, key), False)
+        self.assertEqual(sock._validate_header(header, key), False)
 
     def testReadHeader(self):
         sock = ws.WebSocket()
         sock.sock = HeaderSockMock("data/header01.txt")
         status, header = sock._read_headers()
-        self.assertEquals(status, 101)
-        self.assertEquals(header["connection"], "upgrade")
+        self.assertEqual(status, 101)
+        self.assertEqual(header["connection"], "upgrade")
 
         sock.sock = HeaderSockMock("data/header02.txt")
         self.assertRaises(ws.WebSocketException, sock._read_headers)
@@ -211,152 +207,152 @@ class WebSocketTest(unittest.TestCase):
         sock.set_mask_key(create_mask_key)
         s = sock.sock = HeaderSockMock("data/header01.txt")
         sock.send("Hello")
-        self.assertEquals(s.sent[0], "\x81\x85abcd)\x07\x0f\x08\x0e")
+        self.assertEqual(s.sent[0], six.b("\x81\x85abcd)\x07\x0f\x08\x0e"))
 
         sock.send("こんにちは")
-        self.assertEquals(s.sent[1], "\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc")
+        self.assertEqual(s.sent[1], six.b("\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc"))
 
         sock.send(u"こんにちは")
-        self.assertEquals(s.sent[1], "\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc")
+        self.assertEqual(s.sent[1], six.b("\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc"))
 
     def testRecv(self):
         # TODO: add longer frame data
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
-        s.add_packet("\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc")
+        something = six.b("\x81\x8fabcd\x82\xe3\xf0\x87\xe3\xf1\x80\xe5\xca\x81\xe2\xc5\x82\xe3\xcc")
+        s.add_packet(something)
         data = sock.recv()
-        self.assertEquals(data, "こんにちは")
+        self.assertEqual(data, "こんにちは")
 
-        s.add_packet("\x81\x85abcd)\x07\x0f\x08\x0e")
+        s.add_packet(six.b("\x81\x85abcd)\x07\x0f\x08\x0e"))
         data = sock.recv()
-        self.assertEquals(data, "Hello")
+        self.assertEqual(data, "Hello")
 
     def testInternalRecvStrict(self):
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
-        s.add_packet("foo")
+        s.add_packet(six.b("foo"))
         s.add_packet(socket.timeout())
-        s.add_packet("bar")
+        s.add_packet(six.b("bar"))
         s.add_packet(SSLError("The read operation timed out"))
-        s.add_packet("baz")
+        s.add_packet(six.b("baz"))
         with self.assertRaises(ws.WebSocketTimeoutException):
             data = sock._recv_strict(9)
-        with self.assertRaises(ws.WebSocketTimeoutException):
+        with self.assertRaises(SSLError):
             data = sock._recv_strict(9)
         data = sock._recv_strict(9)
-        self.assertEquals(data, "foobarbaz")
+        self.assertEqual(data, six.b("foobarbaz"))
         with self.assertRaises(ws.WebSocketConnectionClosedException):
             data = sock._recv_strict(1)
 
     def testRecvTimeout(self):
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
-        s.add_packet("\x81")
+        s.add_packet(six.b("\x81"))
         s.add_packet(socket.timeout())
-        s.add_packet("\x8dabcd\x29\x07\x0f\x08\x0e")
+        s.add_packet(six.b("\x8dabcd\x29\x07\x0f\x08\x0e"))
         s.add_packet(socket.timeout())
-        s.add_packet("\x4e\x43\x33\x0e\x10\x0f\x00\x40")
+        s.add_packet(six.b("\x4e\x43\x33\x0e\x10\x0f\x00\x40"))
         with self.assertRaises(ws.WebSocketTimeoutException):
             data = sock.recv()
         with self.assertRaises(ws.WebSocketTimeoutException):
             data = sock.recv()
         data = sock.recv()
-        self.assertEquals(data, "Hello, World!")
+        self.assertEqual(data, "Hello, World!")
         with self.assertRaises(ws.WebSocketConnectionClosedException):
             data = sock.recv()
 
-    @unittest.skipUnless(TEST_FRAGMENTATION, "fragmentation not implemented")
     def testRecvWithSimpleFragmentation(self):
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
         # OPCODE=TEXT, FIN=0, MSG="Brevity is "
-        s.add_packet("\x01\x8babcd#\x10\x06\x12\x08\x16\x1aD\x08\x11C")
+        s.add_packet(six.b("\x01\x8babcd#\x10\x06\x12\x08\x16\x1aD\x08\x11C"))
         # OPCODE=CONT, FIN=1, MSG="the soul of wit"
-        s.add_packet("\x80\x8fabcd\x15\n\x06D\x12\r\x16\x08A\r\x05D\x16\x0b\x17")
+        s.add_packet(six.b("\x80\x8fabcd\x15\n\x06D\x12\r\x16\x08A\r\x05D\x16\x0b\x17"))
         data = sock.recv()
         self.assertEqual(data, "Brevity is the soul of wit")
         with self.assertRaises(ws.WebSocketConnectionClosedException):
             sock.recv()
 
-    @unittest.skipUnless(TEST_FRAGMENTATION, "fragmentation not implemented")
     def testRecvContFragmentation(self):
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
         # OPCODE=CONT, FIN=1, MSG="the soul of wit"
-        s.add_packet("\x80\x8fabcd\x15\n\x06D\x12\r\x16\x08A\r\x05D\x16\x0b\x17")
+        s.add_packet(six.b("\x80\x8fabcd\x15\n\x06D\x12\r\x16\x08A\r\x05D\x16\x0b\x17"))
         self.assertRaises(ws.WebSocketException, sock.recv)
 
-    @unittest.skipUnless(TEST_FRAGMENTATION, "fragmentation not implemented")
     def testRecvWithProlongedFragmentation(self):
         sock = ws.WebSocket()
         s = sock.sock = SockMock()
         # OPCODE=TEXT, FIN=0, MSG="Once more unto the breach, "
-        s.add_packet("\x01\x9babcd.\x0c\x00\x01A\x0f\x0c\x16\x04B\x16\n\x15" \
-                     "\rC\x10\t\x07C\x06\x13\x07\x02\x07\tNC")
+        s.add_packet(six.b("\x01\x9babcd.\x0c\x00\x01A\x0f\x0c\x16\x04B\x16\n\x15" \
+                           "\rC\x10\t\x07C\x06\x13\x07\x02\x07\tNC"))
         # OPCODE=CONT, FIN=0, MSG="dear friends, "
-        s.add_packet("\x00\x8eabcd\x05\x07\x02\x16A\x04\x11\r\x04\x0c\x07" \
-                     "\x17MB")
+        s.add_packet(six.b("\x00\x8eabcd\x05\x07\x02\x16A\x04\x11\r\x04\x0c\x07" \
+                           "\x17MB"))
         # OPCODE=CONT, FIN=1, MSG="once more"
-        s.add_packet("\x80\x89abcd\x0e\x0c\x00\x01A\x0f\x0c\x16\x04")
+        s.add_packet(six.b("\x80\x89abcd\x0e\x0c\x00\x01A\x0f\x0c\x16\x04"))
         data = sock.recv()
-        self.assertEqual(data, "Once more unto the breach, dear friends, " \
-                               "once more")
+        self.assertEqual(
+            data,
+            "Once more unto the breach, dear friends, once more")
         with self.assertRaises(ws.WebSocketConnectionClosedException):
             sock.recv()
 
-    @unittest.skipUnless(TEST_FRAGMENTATION, "fragmentation not implemented")
     def testRecvWithFragmentationAndControlFrame(self):
         sock = ws.WebSocket()
         sock.set_mask_key(create_mask_key)
         s = sock.sock = SockMock()
         # OPCODE=TEXT, FIN=0, MSG="Too much "
-        s.add_packet("\x01\x89abcd5\r\x0cD\x0c\x17\x00\x0cA")
+        s.add_packet(six.b("\x01\x89abcd5\r\x0cD\x0c\x17\x00\x0cA"))
         # OPCODE=PING, FIN=1, MSG="Please PONG this"
-        s.add_packet("\x89\x90abcd1\x0e\x06\x05\x12\x07C4.,$D\x15\n\n\x17")
+        s.add_packet(six.b("\x89\x90abcd1\x0e\x06\x05\x12\x07C4.,$D\x15\n\n\x17"))
         # OPCODE=CONT, FIN=1, MSG="of a good thing"
-        s.add_packet("\x80\x8fabcd\x0e\x04C\x05A\x05\x0c\x0b\x05B\x17\x0c" \
-                     "\x08\x0c\x04")
+        s.add_packet(six.b("\x80\x8fabcd\x0e\x04C\x05A\x05\x0c\x0b\x05B\x17\x0c" \
+                           "\x08\x0c\x04"))
         data = sock.recv()
         self.assertEqual(data, "Too much of a good thing")
         with self.assertRaises(ws.WebSocketConnectionClosedException):
             sock.recv()
-        self.assertEqual(s.sent[0], "\x8a\x90abcd1\x0e\x06\x05\x12\x07C4.,$D" \
-                                    "\x15\n\n\x17")
+        self.assertEqual(
+            s.sent[0],
+            six.b("\x8a\x90abcd1\x0e\x06\x05\x12\x07C4.,$D\x15\n\n\x17"))
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testWebSocket(self):
         s = ws.create_connection("ws://echo.websocket.org/")
-        self.assertNotEquals(s, None)
+        self.assertNotEqual(s, None)
         s.send("Hello, World")
         result = s.recv()
-        self.assertEquals(result, "Hello, World")
+        self.assertEqual(result, "Hello, World")
 
-        s.send("こにゃにゃちは、世界")
+        s.send(u"こにゃにゃちは、世界")
         result = s.recv()
-        self.assertEquals(result, "こにゃにゃちは、世界")
+        self.assertEqual(result, "こにゃにゃちは、世界")
         s.close()
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testPingPong(self):
         s = ws.create_connection("ws://echo.websocket.org/")
-        self.assertNotEquals(s, None)
+        self.assertNotEqual(s, None)
         s.ping("Hello")
         s.pong("Hi")
         s.close()
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
+    @unittest.skipUnless(TEST_SECURE_WS, "wss://echo.websocket.org doesn't work well.")
     def testSecureWebSocket(self):
         if 1:
             import ssl
             s = ws.create_connection("wss://echo.websocket.org/")
-            self.assertNotEquals(s, None)
-            self.assert_(isinstance(s.sock, ssl.SSLSocket))
+            self.assertNotEqual(s, None)
+            self.assertTrue(isinstance(s.sock, ssl.SSLSocket))
             s.send("Hello, World")
             result = s.recv()
-            self.assertEquals(result, "Hello, World")
-            s.send("こにゃにゃちは、世界")
+            self.assertEqual(result, "Hello, World")
+            s.send(u"こにゃにゃちは、世界")
             result = s.recv()
-            self.assertEquals(result, "こにゃにゃちは、世界")
+            self.assertEqual(result, "こにゃにゃちは、世界")
             s.close()
         #except:
         #    pass
@@ -365,17 +361,17 @@ class WebSocketTest(unittest.TestCase):
     def testWebSocketWihtCustomHeader(self):
         s = ws.create_connection("ws://echo.websocket.org/",
                                  headers={"User-Agent": "PythonWebsocketClient"})
-        self.assertNotEquals(s, None)
+        self.assertNotEqual(s, None)
         s.send("Hello, World")
         result = s.recv()
-        self.assertEquals(result, "Hello, World")
+        self.assertEqual(result, "Hello, World")
         s.close()
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testAfterClose(self):
         from socket import error
         s = ws.create_connection("ws://echo.websocket.org/")
-        self.assertNotEquals(s, None)
+        self.assertNotEqual(s, None)
         s.close()
         self.assertRaises(error, s.send, "Hello")
         self.assertRaises(error, s.recv)
@@ -385,7 +381,7 @@ class WebSocketTest(unittest.TestCase):
         """
         key = ws._create_sec_websocket_key()
         u = uuid.UUID(bytes=base64.b64decode(key))
-        self.assertEquals(4, u.version)
+        self.assertEqual(4, u.version)
 
 
 class WebSocketAppTest(unittest.TestCase):
@@ -433,8 +429,8 @@ class WebSocketAppTest(unittest.TestCase):
         self.assertFalse(isinstance(WebSocketAppTest.keep_running_close,
                                     WebSocketAppTest.NotSetYet))
 
-        self.assertEquals(True, WebSocketAppTest.keep_running_open)
-        self.assertEquals(False, WebSocketAppTest.keep_running_close)
+        self.assertEqual(True, WebSocketAppTest.keep_running_open)
+        self.assertEqual(False, WebSocketAppTest.keep_running_close)
 
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testSockMaskKey(self):
@@ -456,17 +452,16 @@ class WebSocketAppTest(unittest.TestCase):
         app.run_forever()
 
         # Note: We can't use 'is' for comparing the functions directly, need to use 'id'.
-        self.assertEquals(WebSocketAppTest.get_mask_key_id, id(my_mask_key_func))
+        self.assertEqual(WebSocketAppTest.get_mask_key_id, id(my_mask_key_func))
 
 
 class SockOptTest(unittest.TestCase):
     @unittest.skipUnless(TEST_WITH_INTERNET, "Internet-requiring tests are disabled")
     def testSockOpt(self):
         sockopt = ((socket.IPPROTO_TCP, socket.TCP_NODELAY, 1),)
-        s = ws.WebSocket(sockopt=sockopt)
-        self.assertNotEquals(s.sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY), 0)
         s = ws.create_connection("ws://echo.websocket.org", sockopt=sockopt)
-        self.assertNotEquals(s.sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY), 0)
+        self.assertNotEqual(s.sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY), 0)
+        s.close()
 
 
 if __name__ == "__main__":
